searchNodes=[{"ref":"Transmog.html","title":"Transmog","type":"module","doc":"Transmog is a module which provides the ability to map keys on nested maps, lists and structs to new values. It is useful for when you have external data that you want to convert into an internal format easily. This recursive transformation is made using an internal concept known as key pairs. Key Pairs Key pairs are a list of two-tuples which represent a mapping from a key in the source map to a key in the destination map. The key mapping does not have to be exhaustive and any values that are skipped are simply added to the result. Examples #=&gt; &quot;credentials&quot;, &quot;first_name&quot; to :identity, :first_name [{[&quot;credentials&quot;, &quot;first_name&quot;], [:identity, :first_name]}] When key pairs are simple they can be represented using an internal DSL which is a dot notation string. These dot notation strings also support atoms if you prefix them with :. Examples #=&gt; Same as the previous example [{&quot;credentials.first_name&quot;, &quot;:identity.:first_name&quot;}] Parsing All of the supported key pair formats implement the Transmog.Parser protocol. Technically if you wanted to add support for a different type then you could implement the protocol for them. Formatting We validate key pairs when they are provided to the main entrypoint of the library, format/2 and format!/2. If the key pairs are not valid then we will let you know with an error. Examples #=&gt; Notice: key paths must be equal length! iex&gt; key_paths = [{&quot;credentials&quot;, &quot;:identity.:first_name&quot;}] iex&gt; Transmog.format(%{&quot;credentials&quot; =&gt; &quot;Mike Rudolph&quot;}, key_paths) {:error, :invalid_key_pairs} iex&gt; key_paths = [{&quot;&quot;, &quot;:last_name&quot;}] iex&gt; Transmog.format(%{}, key_paths) {:error, :invalid_key_path} Once your key pairs are validated you can start to use them to transform your nested maps and lists. Examples #=&gt; Notice: you need to be explicit about which keys you want updated iex&gt; key_paths = [ ...&gt; {&quot;credentials&quot;, &quot;:identity&quot;}, ...&gt; {&quot;credentials.first_name&quot;, &quot;:identity.:first_name&quot;} ...&gt; ] iex&gt; source = %{&quot;credentials&quot; =&gt; %{&quot;first_name&quot; =&gt; &quot;Tom&quot;}} iex&gt; {:ok, result} = Transmog.format(source, key_paths) iex&gt; result %{identity: %{first_name: &quot;Tom&quot;}} iex&gt; key_paths = [ ...&gt; {&quot;credentials&quot;, &quot;:identity&quot;}, ...&gt; {&quot;credentials.first_name&quot;, &quot;:identity.:first_name&quot;} ...&gt; ] iex&gt; source = [ ...&gt; %{&quot;credentials&quot; =&gt; %{&quot;first_name&quot; =&gt; &quot;John&quot;}}, ...&gt; %{&quot;credentials&quot; =&gt; %{&quot;first_name&quot; =&gt; &quot;Sally&quot;}} ...&gt; ] iex&gt; {:ok, result} = Transmog.format(source, key_paths) iex&gt; result [%{identity: %{first_name: &quot;John&quot;}}, %{identity: %{first_name: &quot;Sally&quot;}}] If you know that your key pairs are valid then you can use format!/2 instead and your results will be unwrapped automatically for you. Examples iex&gt; key_paths = [{&quot;name&quot;, &quot;:name&quot;}] iex&gt; source = %{&quot;name&quot; =&gt; &quot;Jimmy&quot;} iex&gt; Transmog.format!(source, key_paths) %{name: &quot;Jimmy&quot;} You can even transform your structs. When a struct is encountered during the parse it will be converted into a map automatically for you."},{"ref":"Transmog.html#format/2","title":"Transmog.format/2","type":"function","doc":"format/2 takes a source value and either a list of key paths or a Transmog.KeyPair struct directly and performs the key transformation on the value. If the key paths are given then they will be parsed using Transmog.KeyPairs.parse/1 and will report errors if any occur during that. Examples iex&gt; key_paths = [{&quot;a&quot;, &quot;:a&quot;}, {&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; source = %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; &quot;c&quot;}} iex&gt; {:ok, result} = Transmog.format(source, key_paths) iex&gt; result %{a: %{b: &quot;c&quot;}} iex&gt; key_paths = [{&quot;a&quot;, &quot;:a&quot;}, {&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; {:ok, %Transmog.KeyPairs{} = key_pairs} = Transmog.KeyPairs.parse(key_paths) iex&gt; source = %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; &quot;c&quot;}} iex&gt; {:ok, result} = Transmog.format(source, key_pairs) iex&gt; result %{a: %{b: &quot;c&quot;}}"},{"ref":"Transmog.html#format!/2","title":"Transmog.format!/2","type":"function","doc":"format!/2 takes a source value and either a list of key paths or a Transmog.KeyPair struct directly and performs the key transformation on the value. This function will raise an error if the Transmog.KeyPair struct cannot be created by parsing the key paths. The result will be automatically unwrapped if the operation is successful. Examples iex&gt; key_paths = [{&quot;a&quot;, &quot;:a&quot;}, {&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; source = %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; &quot;c&quot;}} iex&gt; Transmog.format!(source, key_paths) %{a: %{b: &quot;c&quot;}} iex&gt; key_paths = [{&quot;a&quot;, &quot;:a&quot;}, {&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; %Transmog.KeyPairs{} = key_pairs = Transmog.KeyPairs.parse!(key_paths) iex&gt; source = %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; &quot;c&quot;}} iex&gt; Transmog.format!(source, key_pairs) %{a: %{b: &quot;c&quot;}}"},{"ref":"Transmog.html#t:key_paths/0","title":"Transmog.key_paths/0","type":"type","doc":"key_paths/0 is the type for a single tuple of key paths. Both sides can be any type as long as they can be parsed by Transmog.Parser."},{"ref":"Transmog.InvalidKeyPairsError.html","title":"Transmog.InvalidKeyPairsError","type":"exception","doc":""},{"ref":"Transmog.InvalidKeyPairsError.html#new/1","title":"Transmog.InvalidKeyPairsError.new/1","type":"function","doc":"new/1 raises an error with a custom field added to the error message. Examples iex&gt; Transmog.InvalidKeyPairsError.new(nil) ** (Transmog.InvalidKeyPairsError) key pairs are not valid (nil)"},{"ref":"Transmog.InvalidKeyPairsError.html#new/2","title":"Transmog.InvalidKeyPairsError.new/2","type":"function","doc":"new/2 raises an error with a custom field and index added to the error message. Examples iex&gt; Transmog.InvalidKeyPairsError.new(nil, 1) ** (Transmog.InvalidKeyPairsError) key pairs are not valid (nil, index 1)"},{"ref":"Transmog.InvalidKeyPathError.html","title":"Transmog.InvalidKeyPathError","type":"exception","doc":""},{"ref":"Transmog.InvalidKeyPathError.html#new/1","title":"Transmog.InvalidKeyPathError.new/1","type":"function","doc":"new/1 raises an error with a custom field added to the error message. Examples iex&gt; Transmog.InvalidKeyPathError.new(nil) ** (Transmog.InvalidKeyPathError) key path is not valid (nil)"},{"ref":"Transmog.KeyPairs.html","title":"Transmog.KeyPairs","type":"module","doc":"Transmog.KeyPairs is a struct which holds the information about a list of key_pair/0 and ensures that they are valid. A key pair is a list of mappings from one path to another. For example, {[:a], [:b]} indicates that we are transforming a map with keys :a and :b to now have the keys swapped with the same values. You can create a new Transmog.KeyPairs struct manually by calling the new/1 and new!/1 functions directly. This struct can be used in most of the core functionality in this library. This library uses parse/1 and parse!/1 under the hood to coerce your key paths into a format that can be understood by this struct. These functions use the Transmog.Parser protocol for the type that is provided for the key path. Examples iex&gt; key_pairs = [{[:identity, :first_name], [:user_details, :first_name]}] iex&gt; {:ok, %Transmog.KeyPairs{} = key_pairs} = Transmog.KeyPairs.new(key_pairs) iex&gt; key_pairs %Transmog.KeyPairs{list: [{[:identity, :first_name], [:user_details, :first_name]}]} iex&gt; key_pairs = [{[:identity, :first_name], [:user_details, :first_name]}] iex&gt; Transmog.KeyPairs.new!(key_pairs) %Transmog.KeyPairs{list: [{[:identity, :first_name], [:user_details, :first_name]}]} If you do not provide correct key pairs when this struct is created then you will receive an invalid/0 error as a response instead. Examples iex&gt; key_pairs = [{nil, [:identity, :last_name]}] iex&gt; Transmog.KeyPairs.new(key_pairs) {:error, :invalid_key_pairs} iex&gt; key_paths = [{nil, &quot;:identity.:last_name&quot;}] iex&gt; Transmog.KeyPairs.parse(key_paths) {:error, :invalid_key_path} #=&gt; Also possible to receive these errors iex&gt; key_pairs = [{nil, [:identity, :last_name]}] iex&gt; Transmog.KeyPairs.new!(key_pairs) ** (Transmog.InvalidKeyPairsError) key pairs are not valid ({nil, [:identity, :last_name]}, index 0) iex&gt; key_paths = [{nil, &quot;:identity.:last_name&quot;}] iex&gt; Transmog.KeyPairs.parse!(key_paths) ** (Transmog.InvalidKeyPathError) key path is not valid (nil) If you know the shape of your data structures in advance then you should pre-compile your Transmog.KeyPairs structs by calling parse!/1 or new!/1 and saving the results somewhere where they can be reused. Transmog.format/2 and Transmog.format!/2 optimize for the case where you provide this struct directly."},{"ref":"Transmog.KeyPairs.html#find_match/2","title":"Transmog.KeyPairs.find_match/2","type":"function","doc":"find_match/2 takes a list of key pairs and a key and finds the pair in the list which matches. If a match is found then the opposite key in the key pair will be returned as the key that the value should map to. Examples iex&gt; key_pairs = %Transmog.KeyPairs{list: [{[:a], [&quot;a&quot;]}]} iex&gt; Transmog.KeyPairs.find_match(key_pairs, [:a]) [&quot;a&quot;] iex&gt; key_pairs = %Transmog.KeyPairs{list: [{[:a], [&quot;a&quot;]}]} iex&gt; Transmog.KeyPairs.find_match(key_pairs, [:b]) [:b]"},{"ref":"Transmog.KeyPairs.html#new/1","title":"Transmog.KeyPairs.new/1","type":"function","doc":"new/1 creates a new Transmog.KeyPairs struct. It enforces that the key pairs are valid and have been previously parsed. If the key pairs are not valid then an error will be returned. Examples iex&gt; key_pairs = [{[:a, :b], [:b, :a]}] iex&gt; {:ok, key_pairs} = Transmog.KeyPairs.new(key_pairs) iex&gt; key_pairs %Transmog.KeyPairs{list: [{[:a, :b], [:b, :a]}]} iex&gt; key_pairs = [{nil, [:a, :b]}] iex&gt; Transmog.KeyPairs.new(key_pairs) {:error, :invalid_key_pairs}"},{"ref":"Transmog.KeyPairs.html#new!/1","title":"Transmog.KeyPairs.new!/1","type":"function","doc":"new!/1 creates a new Transmog.KeyPairs struct. It delegates execution to new/1. If the key pairs are not valid then an error is raised otherwise the result is unwrapped and returned. Examples iex&gt; key_pairs = [{[:a, :b], [:b, :a]}] iex&gt; Transmog.KeyPairs.new!(key_pairs) %Transmog.KeyPairs{list: [{[:a, :b], [:b, :a]}]} iex&gt; key_pairs = [{nil, [:a, :b]}] iex&gt; Transmog.KeyPairs.new!(key_pairs) ** (Transmog.InvalidKeyPairsError) key pairs are not valid ({nil, [:a, :b]}, index 0)"},{"ref":"Transmog.KeyPairs.html#parse/1","title":"Transmog.KeyPairs.parse/1","type":"function","doc":"parse/1 takes a list of key paths and attempts to coerce them into valid key pairs. If successful then a Transmog.KeyPair struct will be returned with the key paths converted into the list format. Examples iex&gt; key_paths = [{&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; {:ok, key_pairs} = Transmog.KeyPairs.parse(key_paths) iex&gt; key_pairs %Transmog.KeyPairs{list: [{[&quot;a&quot;, &quot;b&quot;], [:a, :b]}]} iex&gt; key_paths = [{&quot;&quot;, &quot;:a&quot;}, {&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; Transmog.KeyPairs.parse(key_paths) {:error, :invalid_key_path} iex&gt; key_paths = [{&quot;a&quot;, &quot;:a.:b&quot;}] iex&gt; Transmog.KeyPairs.parse(key_paths) {:error, :invalid_key_pairs}"},{"ref":"Transmog.KeyPairs.html#parse!/1","title":"Transmog.KeyPairs.parse!/1","type":"function","doc":"parse!/1 takes a list of key paths are coerces them into valid key pairs. If the key pairs or any path are not valid then an error will be raised. Otherwise the result is unwrapped and returned. Examples iex&gt; key_paths = [{&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; Transmog.KeyPairs.parse!(key_paths) %Transmog.KeyPairs{list: [{[&quot;a&quot;, &quot;b&quot;], [:a, :b]}]} iex&gt; key_paths = [{&quot;&quot;, &quot;:a&quot;}, {&quot;a.b&quot;, &quot;:a.:b&quot;}] iex&gt; Transmog.KeyPairs.parse!(key_paths) ** (Transmog.InvalidKeyPathError) key path is not valid (&quot;&quot;) iex&gt; key_paths = [{&quot;a&quot;, &quot;:a.:b&quot;}] iex&gt; Transmog.KeyPairs.parse!(key_paths) ** (Transmog.InvalidKeyPairsError) key pairs are not valid ({[&quot;a&quot;], [:a, :b]}, index 0)"},{"ref":"Transmog.KeyPairs.html#reverse/1","title":"Transmog.KeyPairs.reverse/1","type":"function","doc":"reverse/1 takes a Transmog.KeyPairs and reverses each individual key pair in the list of key pairs. This could be useful if you wanted to reverse a format without having to define two separate key pairs manually. Examples iex&gt; key_paths = [{&quot;a&quot;, &quot;b&quot;}] iex&gt; %Transmog.KeyPairs{} = key_pairs = Transmog.KeyPairs.parse!(key_paths) iex&gt; Transmog.KeyPairs.reverse(key_pairs) %Transmog.KeyPairs{list: [{[&quot;b&quot;], [&quot;a&quot;]}]}"},{"ref":"Transmog.KeyPairs.html#t:error/0","title":"Transmog.KeyPairs.error/0","type":"type","doc":"error/0 is the type for when creating a new Transmog.Keypair struct fails due to either an invalid path or pair."},{"ref":"Transmog.KeyPairs.html#t:invalid/0","title":"Transmog.KeyPairs.invalid/0","type":"type","doc":"invalid/0 is the type for when a key pair list is determined to not be valid the struct is created using new/1."},{"ref":"Transmog.KeyPairs.html#t:key_pair/0","title":"Transmog.KeyPairs.key_pair/0","type":"type","doc":"key_pair/0 is the type for a single key pair that is part of the list of key pairs that this struct holds."},{"ref":"Transmog.KeyPairs.html#t:t/0","title":"Transmog.KeyPairs.t/0","type":"type","doc":"t/0 is the type for the Transmog.KeyPair struct."},{"ref":"Transmog.Parser.html","title":"Transmog.Parser","type":"protocol","doc":"Transmog.Parser defines how to parse a single key path. Parsing a key path means to transform it into a list which defines how to reach the target value in the nested map or list. Examples &quot;a.:b.c&quot; #=&gt; References a map or list with key path [&quot;a&quot;, :b, &quot;c&quot;] &quot;:a.1&quot; #=&gt; References a map or list with key path [:a, &quot;1&quot;] Currently the following types are supported by this library: binary/0 nonempty_list/1 Examples iex&gt; string = &quot;credentials.:first_name&quot; iex&gt; {:ok, key_path} = Transmog.Parser.parse(string) iex&gt; key_path [&quot;credentials&quot;, :first_name] iex&gt; string = &quot;credentials\\\\.first_name&quot; iex&gt; {:ok, key_path} = Transmog.Parser.parse(string) iex&gt; key_path [&quot;credentials.first_name&quot;] iex&gt; string = &quot;&quot; iex&gt; Transmog.Parser.parse(string) {:error, :invalid_key_path} When you want to use a period character in your key path then you can escape it with backslashes and it will be ignored when parsing the string. Examples string = &quot;credentials\\\\.first_name&quot; Transmog.Parser.parse!(string) #=&gt; Will return [&quot;credentials.first_name&quot;]"},{"ref":"Transmog.Parser.html#parse/1","title":"Transmog.Parser.parse/1","type":"function","doc":"parse/1 will convert a value into a valid key path. If applicable and the value is not valid, then a value of error/0 will be returned instead."},{"ref":"Transmog.Parser.html#parse!/1","title":"Transmog.Parser.parse!/1","type":"function","doc":"parse!/1 will convert a value into a valid key path. If the key path is not valid then an error will be raised. The result will be unwrapped automatically."},{"ref":"Transmog.Parser.html#t:error/0","title":"Transmog.Parser.error/0","type":"type","doc":"error/0 is the type for the error that the parser should return if it encounters a value that is not valid."},{"ref":"Transmog.Parser.html#t:t/0","title":"Transmog.Parser.t/0","type":"type","doc":""},{"ref":"readme.html","title":"README","type":"extras","doc":"transmog Easily perform deep in-place data mappings for keys on lists and maps."},{"ref":"readme.html#table-of-contents","title":"README - Table Of Contents","type":"extras","doc":"Table Of Contents About Installation Getting Started Contributing License Acknowledgements"},{"ref":"readme.html#about","title":"README - About","type":"extras","doc":"transmog is a simple module which allows for in-place data mapping for keys on deeply nested lists and maps. One use case for this library is to convert external data into a shape that is compatible with an internal data structure."},{"ref":"readme.html#installation","title":"README - Installation","type":"extras","doc":"To install transmog add the following to your mix.exs file and run mix deps.get in your project root directory. defp deps do [{:transmog, &quot;~&gt; 0.1&quot;}] end"},{"ref":"readme.html#getting-started","title":"README - Getting Started","type":"extras","doc":"The simplest way to use transmog is to first create a key mapping for your data using the following. defmodule TransmogExample do @key_mapping [ {&quot;account&quot;, &quot;:user&quot;}, {&quot;account.identity&quot;, &quot;:user.:details&quot;}, {&quot;account.identity.first_name&quot;, &quot;:user.:details.:first_name&quot;} ] end This key mapping says that the account key in a map or list of maps will be converted to :user, identity to :details, and first_name to :first_name. The entire chain does not have to be present if you only want to update the lowest level. Once you have a key mapping defined you can then perform the translation using Transmog.format/2. iex&gt; fields = %{&quot;account&quot; =&gt; %{&quot;identity&quot; =&gt; %{&quot;first_name&quot; =&gt; &quot;Billy&quot;}}} iex&gt; Transmog.format(fields, @key_mapping) {:ok, %{user: %{details: %{first_name: &quot;Bobby&quot;}}}} If you are confident that your key mapping is valid then you can unwrap the results by using Transmog.format!/2. iex&gt; fields = %{&quot;account&quot; =&gt; %{&quot;identity&quot; =&gt; %{&quot;first_name&quot; =&gt; &quot;Sally&quot;}}} iex&gt; Transmog.format!(fields, @key_mapping) %{user: %{details: %{first_name: &quot;Bobby&quot;}}}"},{"ref":"readme.html#contributing","title":"README - Contributing","type":"extras","doc":"I would greatly appreciate any contributions to make this project better. Please make sure to follow the below guidelines before getting your hands dirty. Fork the repository Create your branch (git checkout -b my-branch) Commit any changes to your branch Push your changes to your remote branch Open a pull request"},{"ref":"readme.html#license","title":"README - License","type":"extras","doc":"Distributed under the MIT License. See LICENSE for more information."},{"ref":"readme.html#acknowledgements","title":"README - Acknowledgements","type":"extras","doc":"elixir Copyright &copy; 2019 Dylan Aspden"}]